# Based on eq_grapher.rb (C) 2006 jim.foltz@gmail.com
# Modified by jcrubio - gmail.com
#
# HISTORY
# 2006-10-25 - Initial release
#

require "sketchup.rb"

include Math

module CartGraph3D

   @model = Sketchup.active_model
   @entities = @model.entities

   def self.draw_config
      conf = {}
      prompts = ["X-Axis Range", "Y-Axis Range", "Label?"]
      choices = ["", "", "yes|no"]
      defaults = ["-20..20", "-20..20", "yes"]
      ret = UI.inputbox(prompts, defaults, choices, "Setup Grid")
      conf[:xrange] = eval(ret[0])
      conf[:yrange] = eval(ret[1])
      conf[:label] = (ret[2]  == "no" ? false : true)
      draw_grid(conf)
   end

   def self.draw_default
      conf = {:label => true}
      conf[:perspective] = false
      conf[:xrange] = -20..20
      conf[:yrange] = -20..20
      draw_grid(conf)
   end

   def self.draw_grid(conf)
      #@model.active_view.camera.perspective = conf[:perspective]
      g = @entities.add_group
      xrange = conf[:xrange]
      yrange = conf[:yrange]
      xrange.each do |x|
	 g.entities.add_cline([x, yrange.min], [x, yrange.max])
	 g.entities.add_text(x.to_s, [x, 0]) if conf[:label]
      end
      yrange.each do |y|
	 g.entities.add_cline([xrange.min, y], [xrange.max, y])
	 g.entities.add_text(y.to_s, [0, y]) if conf[:label]
      end
      Sketchup.send_action(CMD_VIEW_TOP)
      Sketchup.send_action(CMD_ZOOM_EXTENTS)
   end

   def self.graph
      include Math
      ret = UI.inputbox( ["z = "], ["( ((x-x.floor)*2 - 1)**2 - 1 )/3 + 1 * sin(x/2 - y/6)"], "Enter valid Ruby syntax" )
      model = Sketchup.active_model
      entities = model.active_entities
      g = @entities.add_group

      -20.step(20, 0.4) do |y|
        l = 0
        -20.step(20, 0.4) do |x|
          z = eval(ret[0])
          next if z < -20 or z > 20
          if l == 0
            g.entities.add_cpoint([x, y, z])
          else
            g.entities.add_edges(l, [x, y, z])
          end
          l = [x, y, z]
        end
      end
   end

   filename = File::basename(__FILE__)
   unless file_loaded?(filename)
      add_separator_to_menu("Plugins")
      graph = UI.menu("Plugins").add_submenu("Graph 3D")
      graph.add_item("Draw grid") {CartGraph3D.draw_default}
      graph.add_item("Graph Eq") {CartGraph3D.graph}
      file_loaded(filename)
   end

end
